// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// {@template symbol_button_theme_extension}
/// Symbol button theme extension that extends [ThemeExtension] and
/// implements copyWith, and lerp methods so you don't have to.
/// {@endtemplate}
class SymbolButtonThemeExtension
    extends ThemeExtension<SymbolButtonThemeExtension> {
  /// {@macro symbol_button_theme_extension}
  const SymbolButtonThemeExtension({
    this.disabledStyle,
    this.focusedStyle,
    this.hoveredStyle,
    this.normalStyle,
    this.tappedStyle,
    this.selectedStyle,
  });

  /// Style of this button in disabled state
  final SymbolButtonStyle? disabledStyle;

  /// Style of this button in focused state
  final SymbolButtonStyle? focusedStyle;

  /// Style of this button in hovered state
  final SymbolButtonStyle? hoveredStyle;

  /// Style of this button in normal state
  final SymbolButtonStyle? normalStyle;

  /// Style of this button in tapped state
  final SymbolButtonStyle? tappedStyle;

  /// Style of this button in selected state
  final SymbolButtonStyle? selectedStyle;

  @override
  ThemeExtension<SymbolButtonThemeExtension> copyWith({
    SymbolButtonStyle? disabledStyle,
    SymbolButtonStyle? focusedStyle,
    SymbolButtonStyle? hoveredStyle,
    SymbolButtonStyle? normalStyle,
    SymbolButtonStyle? tappedStyle,
    SymbolButtonStyle? selectedStyle,
  }) =>
      SymbolButtonThemeExtension(
        disabledStyle: disabledStyle ?? this.disabledStyle,
        focusedStyle: focusedStyle ?? this.focusedStyle,
        hoveredStyle: hoveredStyle ?? this.hoveredStyle,
        normalStyle: normalStyle ?? this.normalStyle,
        tappedStyle: tappedStyle ?? this.tappedStyle,
        selectedStyle: selectedStyle ?? this.selectedStyle,
      );

  @override
  ThemeExtension<SymbolButtonThemeExtension> lerp(
    covariant ThemeExtension<SymbolButtonThemeExtension>? other,
    double t,
  ) {
    if (other is! SymbolButtonThemeExtension) {
      return this;
    }
    return SymbolButtonThemeExtension(
      disabledStyle:
          SymbolButtonStyle.lerp(disabledStyle, other.disabledStyle, t),
      focusedStyle: SymbolButtonStyle.lerp(focusedStyle, other.focusedStyle, t),
      hoveredStyle: SymbolButtonStyle.lerp(hoveredStyle, other.hoveredStyle, t),
      normalStyle: SymbolButtonStyle.lerp(normalStyle, other.normalStyle, t),
      tappedStyle: SymbolButtonStyle.lerp(tappedStyle, other.tappedStyle, t),
      selectedStyle:
          SymbolButtonStyle.lerp(selectedStyle, other.selectedStyle, t),
    );
  }
}
